/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.ui.views.planif;

import org.eclipse.core.databinding.conversion.Converter;

import com.patrikdufresne.util.Localized;
import com.planimod.core.ProposedTasks.ProposedWarning;

/**
 * Converter to create a string for a collection of warnings.
 * 
 * @author Patrik Dufresne
 * 
 */
public class ProposedWarningToStringConverter extends Converter {

    /**
     * Used for localization.
     */
    private static Localized localized = Localized.load(ProposedWarningToStringConverter.class);

    /**
     * Empty string (value "").
     */
    private static final Object EMPTY = ""; //$NON-NLS-1$

    /**
     * Create a new converter.
     */
    public ProposedWarningToStringConverter() {
        super(ProposedWarning.class, String.class);
    }

    /**
     * This implementation convert the warning into a description.
     */
    @Override
    public Object convert(Object fromObject) {
        if (fromObject instanceof ProposedWarning) {
            ProposedWarning warning = (ProposedWarning) fromObject;
            switch (warning.getCode()) {
            case ProposedWarning.AS_SENIORITY_TO_WORK_ON_PREFERRED_TEAM:
                return localized.get("ProposedWarningToStringConverter.asSeniorityToWorkOnPreferredTeam"); //$NON-NLS-1$
            default:
                return localized.get("ProposedWarningToStringConverter.default"); //$NON-NLS-1$
            }
        }
        return EMPTY;
    }
}
